@@ -4,8 +4,8 @@
  * @brief logging routines
  *
  * @copyright
- * Copyright (c) 2018 - 2021 Deutsche Telekom AG.
- * Copyright (c) 2018 - 2021 CESNET, z.s.p.o.
+ * Copyright (c) 2018 - 2024 Deutsche Telekom AG.
+ * Copyright (c) 2018 - 2024 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -16,6 +16,7 @@
 
 #define _GNU_SOURCE
 
+#include "compat.h"
 #include "log.h"
 
 #include <assert.h>
@@ -27,13 +28,12 @@
 
 #include <libyang/libyang.h>
 
-#include "compat.h"
 #include "config.h"
 
-sr_log_level_t stderr_ll = SR_LL_NONE;  /**< stderr log level */
-sr_log_level_t syslog_ll = SR_LL_NONE;  /**< syslog log level */
-int syslog_open;                        /**< Whether syslog was opened */
-sr_log_cb log_cb;                       /**< Logging callback */
+sr_log_level_t sr_stderr_ll = SR_LL_NONE;   /**< stderr log level */
+sr_log_level_t sr_syslog_ll = SR_LL_NONE;   /**< syslog log level */
+int syslog_open;                            /**< Whether syslog was opened */
+sr_log_cb sr_lcb;                           /**< Logging callback */
 
 /**
  * @brief String error list.
@@ -79,7 +79,7 @@ static sr_error_info_t sr_errinfo_mem = {
     .err_count = 1
 };
 
-sr_error_t
+int
 sr_api_ret(sr_session_ctx_t *session, sr_error_info_t *err_info)
 {
     sr_error_t err_code = SR_ERR_OK;
@@ -132,18 +132,18 @@ sr_log_msg(int plugin, sr_log_level_t ll, const char *msg)
     }
 
     /* stderr logging */
-    if (ll <= stderr_ll) {
-        fprintf(stderr, "[%s]%s %s\n", severity, plugin ? " PLG:" : "", msg);
+    if (ll <= sr_stderr_ll) {
+        fprintf(stderr, "[%s] %s\n", severity, msg);
     }
 
     /* syslog logging */
-    if (ll <= syslog_ll) {
+    if (ll <= sr_syslog_ll) {
         syslog(priority | (plugin ? LOG_DAEMON : 0), "[%s] %s\n", severity, msg);
     }
 
     /* logging callback */
-    if (log_cb) {
-        log_cb(ll, msg);
+    if (sr_lcb) {
+        sr_lcb(ll, msg);
     }
 }
 
@@ -224,6 +224,10 @@ sr_errinfo_new(sr_error_info_t **err_info, sr_error_t err_code, const char *msg_
         /* there is no dynamic memory, use the static error structure */
         sr_errinfo_free(err_info);
         *err_info = &sr_errinfo_mem;
+    } else if (!msg_format) {
+        /* error without a message */
+        sr_errinfo_add(err_info, err_code, NULL, NULL, NULL, NULL);
+        return;
     } else {
         va_start(vargs, msg_format);
         sr_errinfo_add(err_info, err_code, NULL, NULL, msg_format, &vargs);
@@ -257,85 +261,14 @@ sr_errinfo_new_data(sr_error_info_t **err_info, sr_error_t err_code, const char
     sr_log_msg(0, SR_LL_ERR, (*err_info)->err[idx].message);
 }
 
-void
-sr_errinfo_new_ly(sr_error_info_t **err_info, const struct ly_ctx *ly_ctx)
-{
-    struct ly_err_item *e;
-
-    e = ly_err_first(ly_ctx);
-
-    /* this function is called only when an error is expected, but it is still possible there
-     * will be none -> libyang problem or simply the error was externally processed, sysrepo is
-     * unable to detect that */
-    if (!e) {
-        sr_errinfo_new(err_info, SR_ERR_LY, "Unknown libyang error.");
-        return;
-    }
-
-    do {
-        if (e->level == LY_LLWRN) {
-            /* just print it */
-            sr_log_msg(0, SR_LL_WRN, e->msg);
-        } else {
-            assert(e->level == LY_LLERR);
-            /* store it and print it */
-            sr_errinfo_new(err_info, SR_ERR_LY, e->msg);
-        }
-
-        e = e->next;
-    } while (e);
-
-    ly_err_clean((struct ly_ctx *)ly_ctx, NULL);
-}
-
-void
-sr_errinfo_new_ly_first(sr_error_info_t **err_info, const struct ly_ctx *ly_ctx)
-{
-    struct ly_err_item *e;
-
-    e = ly_err_first(ly_ctx);
-    /* this function is called only when an error is expected */
-    assert(e);
-
-    if (e->level == LY_LLWRN) {
-        /* just print it */
-        sr_log_msg(0, SR_LL_WRN, e->msg);
-    } else {
-        assert(e->level == LY_LLERR);
-        /* store it and print it */
-        sr_errinfo_new(err_info, SR_ERR_LY, e->msg);
-    }
-
-    ly_err_clean((struct ly_ctx *)ly_ctx, NULL);
-}
-
-void
-sr_log_wrn_ly(const struct ly_ctx *ly_ctx)
-{
-    struct ly_err_item *e;
-
-    e = ly_err_first(ly_ctx);
-    /* this function is called only when an error is expected */
-    assert(e);
-
-    do {
-        /* print everything as warnings */
-        sr_log_msg(0, SR_LL_WRN, e->msg);
-
-        e = e->next;
-    } while (e);
-
-    ly_err_clean((struct ly_ctx *)ly_ctx, NULL);
-}
-
 void
 sr_errinfo_free(sr_error_info_t **err_info)
 {
     size_t i;
 
     if (err_info && *err_info) {
-        /* NO_MEM is always a static error info structure */
-        if (((*err_info)->err_count != 1) || ((*err_info)->err[0].err_code != SR_ERR_NO_MEMORY)) {
+        /* NO_MEM is a static error info structure */
+        if (*err_info != &sr_errinfo_mem) {
             for (i = 0; i < (*err_info)->err_count; ++i) {
                 free((*err_info)->err[i].message);
                 free((*err_info)->err[i].error_format);
@@ -389,6 +322,52 @@ sr_log(sr_log_level_t ll, const char *format, ...)
     free(msg);
 }
 
+API void
+srplg_log_errinfo(sr_error_info_t **err_info, const char *plg_name, const char *err_format_name, sr_error_t err_code,
+        const char *format, ...)
+{
+    va_list vargs;
+    char *msg;
+    int idx;
+
+    if (!plg_name) {
+        return;
+    }
+
+    /* add plugin name first */
+    if (asprintf(&msg, "%s: %s", plg_name, format) == -1) {
+        *err_info = &sr_errinfo_mem;
+    } else {
+        /* add err_info */
+        va_start(vargs, format);
+        sr_errinfo_add(err_info, err_code, err_format_name, NULL, msg, &vargs);
+        va_end(vargs);
+    }
+
+    /* print it */
+    idx = (*err_info)->err_count - 1;
+    sr_log_msg(1, SR_LL_ERR, (*err_info)->err[idx].message);
+    free(msg);
+}
+
+API int
+srplg_errinfo_push_error_data(sr_error_info_t *err_info, uint32_t size, const void *data)
+{
+    sr_error_info_t *einfo = NULL;
+
+    SR_CHECK_ARG_APIRET(!err_info || !err_info->err_count || !err_info->err[err_info->err_count - 1].error_format ||
+            !size || !data, NULL, einfo);
+
+    einfo = sr_ev_data_push(&err_info->err[err_info->err_count - 1].error_data, size, data);
+    return sr_api_ret(NULL, einfo);
+}
+
+API void
+srplg_errinfo_free(sr_error_info_t **err_info)
+{
+    sr_errinfo_free(err_info);
+}
+
 API void
 srplg_log(const char *plg_name, sr_log_level_t ll, const char *format, ...)
 {
@@ -412,21 +391,6 @@ srplg_log(const char *plg_name, sr_log_level_t ll, const char *format, ...)
     free(msg);
 }
 
-API void
-srp_log(sr_log_level_t ll, const char *format, ...)
-{
-    va_list ap;
-    char *msg;
-    int msg_len = 0;
-
-    va_start(ap, format);
-    sr_vsprintf(&msg, &msg_len, 0, format, ap);
-    va_end(ap);
-
-    sr_log_msg(1, ll, msg);
-    free(msg);
-}
-
 API const char *
 sr_strerror(int err_code)
 {
@@ -440,25 +404,19 @@ sr_strerror(int err_code)
 API void
 sr_log_stderr(sr_log_level_t log_level)
 {
-    /* initializes libyang logging for our purpose */
-    ly_log_options(LY_LOSTORE);
-
-    stderr_ll = log_level;
+    sr_stderr_ll = log_level;
 }
 
 API sr_log_level_t
 sr_log_get_stderr(void)
 {
-    return stderr_ll;
+    return sr_stderr_ll;
 }
 
 API void
 sr_log_syslog(const char *app_name, sr_log_level_t log_level)
 {
-    /* initializes libyang logging for our purpose */
-    ly_log_options(LY_LOSTORE);
-
-    syslog_ll = log_level;
+    sr_syslog_ll = log_level;
 
     if ((log_level > SR_LL_NONE) && !syslog_open) {
         openlog(app_name ? app_name : "sysrepo", LOG_CONS | LOG_PID | LOG_NDELAY, LOG_USER);
@@ -474,11 +432,11 @@ sr_log_syslog(const char *app_name, sr_log_level_t log_level)
 API sr_log_level_t
 sr_log_get_syslog(void)
 {
-    return syslog_ll;
+    return sr_syslog_ll;
 }
 
 API void
 sr_log_set_cb(sr_log_cb log_callback)
 {
-    log_cb = log_callback;
+    sr_lcb = log_callback;
 }
